using System;
using System.Collections.Generic;
using System.Text;

namespace RecurentSeq
{
	public static class ConsoleProgram
	{
		/// <summary> Перетворює стрічкове представлення числа в число. </summary>
		/// <param name="s"> Стрічка, яка містить одне число. </param>
		/// <returns> Число з плаваючою крапкою. </returns>
		/// <exception cref="System.ArgumentNullException">
		/// Параметр <paramref name="s"/> має значення null.
		/// </exception>
		/// <exception cref="System.FormatException">
		/// Параметр <paramref name="s"/> не є числом допустимого формату.
		/// </exception>
		/// <exception cref="System.OverflowException">
		/// Параметр <paramref name="s"/> представляє число,
		/// яке є меншим ніж <see cref="System.Double.MinValue"/>,
		/// або більшим ніж <see cref="System.Double.MaxValue"/>.
		/// </exception>
		static double DoubleParse(string s)
		{
			double value;
			try { value = double.Parse(s, System.Globalization.CultureInfo.CurrentCulture); }
			catch(FormatException)
			{
				try { value = double.Parse(s, System.Globalization.CultureInfo.InvariantCulture); }
				catch(FormatException)
				{
					value = double.Parse(s.Replace(',', '.'), System.Globalization.CultureInfo.InvariantCulture);
				}
			}
			return value;
		}
		static uint UintParse(string s)
		{
			uint value;
			try { value = uint.Parse(s, System.Globalization.CultureInfo.CurrentCulture); }
			catch(FormatException)
			{
				value = uint.Parse(s, System.Globalization.CultureInfo.InvariantCulture);
			}
			return value;
		}

		static double GetDoubleFromConsole(string write)
		{
			double? result = null;
			do
			{
				Console.Write('\r');
				Console.Write(write);
				for(int i = write.Length; ++i < Console.BufferWidth; )
					Console.Write(' ');
				Console.Write('\r');
				Console.Write(write);
				try { result = DoubleParse(Console.ReadLine()); }
				catch(Exception) { --Console.CursorTop; }
			}
			while(!result.HasValue);
			return result.Value;
		}
		static uint GetIntFromConsole(string write)
		{
			uint? result = null;
			do
			{
				Console.Write('\r');
				Console.Write(write);
				for(int i = write.Length; ++i < Console.BufferWidth; )
					Console.Write(' ');
				Console.Write('\r');
				Console.Write(write);
				try { result = UintParse(Console.ReadLine()); }
				catch(Exception) { --Console.CursorTop; }
			}
			while(!result.HasValue);
			return result.Value;
		}
		static void Main(string[] args)
		{
			Console.Write("\r\n");
			double a1 = GetDoubleFromConsole(" a[1] = ");
			double a2 = GetDoubleFromConsole(" a[2] = ");
			double an;

			Console.Write("\r\n a[n] = A * a[n-1] + B * a[n-2]\r\n\n");
			double a = GetDoubleFromConsole(" A = ");
			double b = GetDoubleFromConsole(" B = ");
			Console.WriteLine();
			uint n = 0;
			while(n < 10)
			{
				Console.WriteLine(" a[{0}] = {1}", n, MathSeq.X_n(a1, a2, a, b, n++));
			}
			Console.WriteLine();
			n = GetIntFromConsole(" n = ");
			Console.WriteLine(" a[{0}] = {1}", n, MathSeq.X_n(a1, a2, a, b, n));

			//Console.WriteLine(" F[n] = {0}", MathSeq.Fibonacci(n));
			//Console.WriteLine(" L[n] = {0}", MathSeq.Lucas(n));

			Console.Write("\r\n Press enter to exit.");
			Console.ReadLine();
		}
	}
}
