// IntExtensions

using System;
using System.Collections.Generic;
using System.Text;

namespace RecurentSeq
{
	public static partial class IntExtensions
	{
		public static ulong Pow(this uint value, uint power)
		{
			ulong r = 1uL, m = value;
			while(true)
			{
				if((power & 1) != 0) r *= m;
				if((power >>= 1) == 0) return r;
				m *= m;
			}
		}
		public static long Pow(this int value, uint power)
		{
			long r = 1L, m = value;
			while(true)
			{
				if((power & 1) != 0) r *= m;
				if((power >>= 1) == 0) return r;
				m *= m;
			}
		}
		public static ulong Pochhammer(this uint value, uint n) { var r = 1uL; while(n > 0u) r *= value + --n; return r; }
		public static long Pochhammer(this int value, uint n) { var r = 1L; while(n > 0u) r *= value + --n; return r; }
		public static ulong Pochhammer(this ulong value, ulong n) { var r = 1uL; while(n > 0u) r *= value + --n; return r; }
		public static ulong Factorial(this ulong value) { var r = 1uL; while(value > 1u) r *= value--; return r; }
		public static ulong Factorial(this uint value) { var r = 1uL; while(value > 1u) r *= value--; return r; }
		public static ulong Factorial(this ushort value) { var r = 1uL; while(value > 1u) r *= value--; return r; }
		public static ulong Factorial(this byte value) { var r = 1uL; while(value > 1u) r *= value--; return r; }
		public static long Factorial(this long value) { var r = 1L; while(value > 1u) r *= value--; return r; }
		public static long Factorial(this int value) { var r = 1L; while(value > 1u) r *= value--; return r; }
		public static long Factorial(this short value) { var r = 1L; while(value > 1u) r *= value--; return r; }
		public static long Factorial(this sbyte value) { var r = 1L; while(value > 1u) r *= value--; return r; }
		public static ulong Combination(this uint n, uint k) { return (n - k + 1u).Pochhammer(k) / k.Factorial(); }
		public static double Combination(this uint n, double k) { return (n - k + 1d).Pochhammer(k) / (++k).Gamma(); }
	}
}
