using System;

namespace RecurentSeq
{
	public static class MathSeq
	{
		/// <summary> Sqrt(5) </summary>
		public const double Sqrt5 = 2.23606797749978969641d;
		/// <summary> (1 + Sqrt(5)) / 2 </summary>
		public const double GoldenRatio = 1.6180339887498948482d;

		/// <summary>
		/// F[0] = 0; F[1] = 1; F[2] = 1; F[3] = 2; F[4] = 3; ...
		/// F[n] = F[n - 1] + F[n - 2].
		/// </summary>
		public static double Fibonacci(double n)
		{
			double pow = Math.Pow(GoldenRatio, n);
			return (pow - Math.Cos(n * Math.PI) / pow) / Sqrt5;
		}
		/// <summary>
		/// L[0] = 2; L[1] = 1; L[2] = 3; L[3] = 4; L[4] = 7; ...
		/// L[n] = L[n - 1] + L[n - 2].
		/// </summary>
		public static double Lucas(double n)
		{
			double pow = Math.Pow(GoldenRatio, n);
			return (pow + Math.Cos(n * Math.PI) / pow);
		}
		/// <summary>
		/// X[1] = x1; X[2] = x2; X[3] = a * x2 + b * x1; ...
		/// X[n] = a * X[n - 1] + b * X[n - 2].
		/// </summary>
		public static double X_n(double x1, double x2, double a, double b, uint n)
		{
			double product, sum1 = n > 0u ? 0d : 1d / b,
				sum2 = n > 0u ? a.Pow(1L - n) * b.Pow(n - 1L) * (2d - n).Pochhammer(n - 1L) / (n - 1u).Factorial() : 0d;
			for(long k = 1u; k < n; )
			{
				product = a.Pow(k - n + k) * b.Pow(n - ++k) / (n - k).Factorial();
				sum1 += product * ((ulong)(--k - n + ++k)).Pochhammer((ulong)(n - k));
				sum2 += product * a * ((ulong)(k - n + k)).Pochhammer((ulong)(n - k));
			}
			return (x2 - a * x1) * sum1 + x1 * sum2;
		}
	}
}
